#//-----------------------------------------------
#//TUSB-S01VIN1用サンプルプログラム
#//Python 用モジュール
#//
#//2024_07_24　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

import TUSBSVI1
from functools import partial
from SpFuncs import *
import sys

from tkinter import filedialog

class Tusbs01vin1_Device:
    """TUSB-S01ACC デバイス操作クラス フォーム"""
    DevId = None                    # デバイスID
    DFile = None                    # データ保存ファイル
    smpl_t = None                   # サンプリングタイマー
    SmplStopFlg = False             # サンプリング停止要求
    SmplStatus = 0                  # サンプリングのステート
    SampledLen = 0                  # 取得済みデータ数

    def __init__(self,DevId):
        """イニシャライザ
        
        Arguments:
        DevId           ----    Device Id 0~15
        """
        self.formgen()
        ret = TUSBSVI1.Tusbs01vin1_Device_Open(DevId)                # デバイスオープン
        if ret == 0:    # 正常オープン
            self.DevId = DevId                                      # デバイスIDをクラスメンバ変数に保持する
            self.form.protocol("WM_DELETE_WINDOW",self.DevClose)
            self.EnableDisableChange("Stop")                        # 表示状態を初期化
            self.smpl_t = intTT(0.1,self.SampleT)                   # サンプリングタイマー
            self.smpl_t.Start()
        else:
            ShowInfo(TUSBSVI1.GetErrMessage(ret))                   # 失敗エラーメッセージ表示
            self.form.destroy()                                     # フォームを閉じる 

    def DevClose(self):
        """デバイスを閉じる時の処理"""
        if self.DevId != None:
            self.smpl_t.Stop()
            TUSBSVI1.Tusbs01vin1_Device_Close(self.DevId)            # デバイスクローズ
            self.form.destroy()                                     # フォームを閉じる
            self.DevId = None
    
    def EnableDisableChange(self,stat):
        """ウィジットの表示状態を変える
        
        Arguments:
        stat            ----    設定状態を表す文字列

        'Stop' - デバイスが接続されていて連続取り込み実行していない時
        'Run' - デバイスが接続されていて連続取り込みを実行している時
        """
        EnStop = tk.NORMAL if( stat == 'Stop' ) else tk.DISABLED
        EnRun = tk.NORMAL if( stat == 'Run' ) else tk.DISABLED
        self.BtnTmSet['state'] = EnStop
        self.BtnInSet['state'] = EnStop
        self.BtnSmplStart['state'] = EnStop
        self.BtnSmplStop['state'] = EnRun
        self.BtnSoftTrg['state'] = EnRun
        self.SpbxTimM['state'] = EnStop
        self.SpbxTimN['state'] = EnStop
        self.CmbbGain['state'] = EnStop
        self.CmbbTrgType['state'] = EnStop
        self.SpbxTrgLev['state'] = EnStop
        self.SpbxPreTrgLen['state'] = EnStop
        self.SpbxBufferLen['state'] = EnStop
        if stat != 'Stop' :
            self.form.protocol('WM_DELETE_WINDOW',lambda: 'pass' )
        else :
            self.form.protocol("WM_DELETE_WINDOW",self.DevClose)

    def SamplingStop(self):
        """取り込み停止処理"""
        ret = TUSBSVI1.Tusbs01vin1_Sampling_Stop(self.DevId)     # 取り込み停止
        if ret != 0:
            ShowInfo(TUSBSVI1.GetErrMessage(ret))
        else:
            self.TxtboxStat['text'] = "<Sampling> Stop"
        # 最終データの保存
        while True:
            ret = TUSBSVI1.Tusbs01vin1_Memory_Read(self.DevId,100000)    # サンプリング済みデータ読み出し
            if ret[0] == 12:    # 連続取り込みデータなし
                break
            elif ret[0] != 0:
                ShowInfo(TUSBSVI1.GetErrMessage(ret[0]))
                break
            else:
                leng = len(ret[1])
                for dat in ret[1]:
                    self.DFile.write(str(dat)+"\n")
            if (ret[0] != 0) or (leng == 0):
                break
        self.DFile.close()
        self.EnableDisableChange("Stop")
        self.SmplStatus = 0 # サンプリング状態を停止
    
    # ボタン等ウィジットの操作イベントハンドラ ----------------------
    def BtnTmSet_on(self):
        ret = TUSBSVI1.Tusbs01vin1_Timer_Set(self.DevId,self.SpbxTimNV.get(),self.SpbxTimMV.get())
        if ret != 0:
            ShowInfo(TUSBSVI1.GetErrMessage(ret))

    def BtnInSet_on(self):
        ret = TUSBSVI1.Tusbs01vin1_Input_Set(self.DevId,self.CmbbGain.current())
        if ret != 0:
            ShowInfo(TUSBSVI1.GetErrMessage(ret))
    
    def BtnSoftTrg_on(self):
        ret = TUSBSVI1.Tusbs01vin1_Sampling_Trigger(self.DevId)
        if ret != 0:
            ShowInfo(TUSBSVI1.GetErrMessage(ret))

    def BtnSmplStart_on(self):
        filename = filedialog.asksaveasfilename(
            title = "保存ファイルを指定してください",
            filetypes = [("データファイル","*.csv")],
            defaultextension="csv"
        )
        if len(filename) > 0:   # ファイルがあるかどうか
            try:
                self.DFile = open(filename,"w") # ファイルを開く
            except OSError as e:
                ShowInfo(e)
                return
        Trigger = self.CmbbTrgType.current()
        Level = self.SpbxTrgLevV.get()
        PreLen = self.SpbxPreTrgLenV.get()
        TotalLen = self.SpbxBufferLenV.get()
        ret = TUSBSVI1.Tusbs01vin1_Sampling_Start(self.DevId,Trigger,Level,PreLen,TotalLen)  # サンプリング開始
        if ret != 0:
            ShowInfo(TUSBSVI1.GetErrMessage(ret))
            return
        self.TxtboxStat['text'] = "Waiting for trigger."
        self.EnableDisableChange("Run")
        self.SmplStopFlg = False
        self.SmplStatus = 1
        self.SampledLen = 0
    
    def BtnSmplStop_on(self):
        self.SmplStopFlg = True         # サンプリング中止指示

    # timer -------------------------------------
    def SampleT(self):
        """サンプリングタイマーコールバック"""
        if self.SmplStatus == 0:    # 停止状態
            return
        elif self.SmplStatus == 1:  # トリガ待ち状態
            if self.SmplStopFlg:    # 停止要求
                self.SamplingStop()       # 取り込み停止
            else:
                ret = TUSBSVI1.Tusbs01vin1_Status_Read(self.DevId)   # 取り込み状態確認
                if ret[0] != 0:
                    ShowInfo(TUSBSVI1.GetErrMessage(ret[0]))
                    self.SamplingStop()      # 取り込み停止
                else:
                    status = ret[1]
                    if status == 0: # 何等かの理由で停止状態になった
                        self.SamplingStop() # 取り込み停止
                    elif status == 2:   # サンプリング中になった
                        self.TxtboxStat['text'] = "<Sampling>"
                        self.SmplStatus = 2 # サンプリング状態 2
        elif self.SmplStatus == 2:  # 取り込み中
            if self.SmplStopFlg:    # 停止要求
                self.SamplingStop() # 取り込み停止
            else:
                ret = TUSBSVI1.Tusbs01vin1_Status_Read(self.DevId)   # 取り込み状態確認
                if ret[0] != 0:
                    ShowInfo(TUSBSVI1.GetErrMessage(ret[0]))
                    self.SamplingStop() # 取り込み停止
                else:
                    status = ret[1]
                    leng = ret[3]
                    if status == 0: # 何等かの理由で停止状態になった
                        self.SamplingStop() # 取り込み停止
                    elif status == 2:   # サンプリング中
                        if leng > 0:    # データがあれば
                            ret = TUSBSVI1.Tusbs01vin1_Memory_Read(self.DevId,leng)  # サンプリング済みデータ読み出し
                            if ret[0] != 0:
                                self.SamplingStop() # 取り込み停止
                            else:
                                leng = len(ret[1])
                                for dat in ret[1]:
                                    self.DFile.write(str(dat)+"\n")
                                self.SampledLen += leng
                                self.TxtboxStat['text'] = str(self.SampledLen)
        
    # フォームとウィジェットを生成する処理---------------------------
    def formgen(self):
        self.form = tk.Tk()
        form = self.form
        form.geometry('401x418')
        form.title('TUSB-S01VI1 Sample Program')

        self.BtnTmSet = GetButton(form,"TimerSet",self.BtnTmSet_on,0.2,0.2,0.1)
        GetLabel(form,"M",0.4,0.05)
        GetLabel(form,"N",0.7,0.05)
        self.SpbxTimMV = tk.IntVar(value=480)
        self.SpbxTimM = GetSpinBoxN(form,1,65536,1,self.SpbxTimMV,0.15,0.4,0.1)
        GetLabel(form,"x10 ^",0.6,0.1)
        self.SpbxTimNV = tk.IntVar(value=0)
        self.SpbxTimN = GetSpinBoxN(form,0,4,1,self.SpbxTimNV,0.15,0.7,0.1)

        GainDict ={
            "x1":0,
            "x10":1,
            "x100":2,
        }
        self.BtnInSet = GetButton(form,"InputSet",self.BtnInSet_on,0.2,0.2,0.25)
        GetLabel(form,"Gain",0.4,0.2)
        self.CmbbGain = GetComboBox(form,list(GainDict.keys()),0.15,0.4,0.25)

        TrgTypeDict ={
            "Soft":0,
            "Upper":1,
            "Under":2,
            "Abs Level":3,
            "Up Edge":4,
            "Down Edge":5,
        }
        GetLabel(form,"TriggerType",0.1,0.35)
        self.CmbbTrgType = GetComboBox(form,list(TrgTypeDict.keys()),0.2,0.1,0.4)
        GetLabel(form,"TriggerLevel",0.4,0.35)
        self.SpbxTrgLevV = tk.IntVar(value=0)
        self.SpbxTrgLev = GetSpinBoxN(form,-32768,32767,1,self.SpbxTrgLevV,0.15,0.4,0.4)
        self.BtnSoftTrg = GetButton(form,"SoftTrigger",self.BtnSoftTrg_on,0.2,0.775,0.4)

        GetLabel(form,"PreTriggerLen",0.4,0.5)
        GetLabel(form,"BufferLen",0.7,0.5)
        self.SpbxPreTrgLenV = tk.IntVar(value=0)
        self.SpbxBufferLenV = tk.IntVar(value=1000)
        self.SpbxPreTrgLen = GetSpinBoxN(form,0,10000000,1,self.SpbxPreTrgLenV,0.15,0.4,0.55)
        self.SpbxBufferLen = GetSpinBoxN(form,0,10000000,1,self.SpbxBufferLenV,0.15,0.7,0.55)

        self.BtnSmplStart = GetButton(form,"StartSampling",self.BtnSmplStart_on,0.3,0.25,0.65)
        self.BtnSmplStop = GetButton(form,"StopSampling",self.BtnSmplStop_on,0.3,0.75,0.65)

        self.TxtboxStat = GetLabelDispML(form,"",0.8,0.2,0.1,0.7)

# 実行部 ---------------------------------
if __name__=="__main__":
    DevId = 0
    if( len(sys.argv) >= 2 ):
        if sys.argv[1].isdigit():
            DevId = int(sys.argv[1])    # コマンドライン引数でID設定
    tmp = Tusbs01vin1_Device(DevId=DevId)
    tmp.form.mainloop()

